#ifndef _ASM_X86_SPEC_CTRL_H
#define _ASM_X86_SPEC_CTRL_H

#define SPEC_CTRL_PCP_IBRS_ENTRY	(1<<0)
#define SPEC_CTRL_PCP_IBRS_EXIT		(1<<1)

#define SPEC_CTRL_PCP_IBRS (SPEC_CTRL_PCP_IBRS_ENTRY|SPEC_CTRL_PCP_IBRS_EXIT)

#define FEATURE_ENABLE_IBRS	(1<<0)
#define FEATURE_SET_IBPB	(1<<0)
#define FEATURE_ENABLE_SSBD_SHIFT	2
#define FEATURE_ENABLE_SSBD	(1<<FEATURE_ENABLE_SSBD_SHIFT)  /* Reduced Data Speculation */

#ifdef __ASSEMBLY__

#include <asm/msr.h>

#define PER_CPU_VAR(x)	%gs:((x))
#define IBRS_ENABLED_PCP	PER_CPU_VAR(KERNEL_IBRS_SPEC_CTRL_enabled)
#define IBRS_ENTRY_PCP		PER_CPU_VAR(KERNEL_IBRS_SPEC_CTRL_entry)
#define IBRS_EXIT_PCP		PER_CPU_VAR(KERNEL_IBRS_SPEC_CTRL_exit)
#define IBRS_HI32_PCP		PER_CPU_VAR(KERNEL_IBRS_SPEC_CTRL_hi32)

.macro __IBRS_ENTRY
	movl IBRS_HI32_PCP, %edx
	movl IBRS_ENTRY_PCP, %eax
	GET_THREAD_INFO(%rcx)
	bt   $TIF_SSBD, threadinfo_flags(%rcx)
	jnc  .Lno_ssbd_\@
	orl  $FEATURE_ENABLE_SSBD, %eax
.Lno_ssbd_\@:
	movl $MSR_IA32_SPEC_CTRL, %ecx
	wrmsr
.endm

.macro IBRS_ENTRY
	testl $SPEC_CTRL_PCP_IBRS_ENTRY, IBRS_ENABLED_PCP
	jz .Lskip_\@

	pushq %rax
	pushq %rcx
	pushq %rdx
	__IBRS_ENTRY
	popq %rdx
	popq %rcx
	popq %rax
	jmp .Lend_\@

.Lskip_\@:
	lfence
.Lend_\@:
.endm

.macro IBRS_ENTRY_CLOBBER
	testl $SPEC_CTRL_PCP_IBRS_ENTRY, IBRS_ENABLED_PCP
	jz .Lskip_\@

	__IBRS_ENTRY
	jmp .Lend_\@

.Lskip_\@:
	lfence
.Lend_\@:
.endm

#define NO_IBRS_RESTORE		(-1)	/* No restore on exit */
/*
 * The save_reg is initialize to NO_IBRS_RESTORE just in case IBRS is
 * enabled in the middle of an exception, this avoids the very remote risk
 * of writing random save_reg content into the SPEC_CTRL MSR in such case.
 */
.macro IBRS_ENTRY_SAVE_AND_CLOBBER save_reg:req
	movl $NO_IBRS_RESTORE, \save_reg
	testl $SPEC_CTRL_PCP_IBRS_ENTRY, IBRS_ENABLED_PCP
	jz .Lskip_\@

	movl $MSR_IA32_SPEC_CTRL, %ecx
	rdmsr

	/*
	 * If the content of the MSR matches the kernel entry value,
	 * we can just leave. %edx is initialized by rdmsr above, and
	 * we don't need to change it. The state of the SSBD bit in
	 * the MSR is transferred to the new value, if necessary.
	 */
	movl IBRS_ENTRY_PCP, %ecx
	cmpl %eax, %ecx
	je   .Lend_\@

	movl %eax, \save_reg
	andl $FEATURE_ENABLE_SSBD, %eax
	orl  %ecx, %eax
	movl $MSR_IA32_SPEC_CTRL, %ecx
	wrmsr
	jmp  .Lend_\@
.Lskip_\@:
	lfence
.Lend_\@:
.endm

.macro __IBRS_EXIT
	movl IBRS_HI32_PCP, %edx
        movl IBRS_EXIT_PCP, %eax
	GET_THREAD_INFO(%rcx)
	bt   $TIF_SSBD, threadinfo_flags(%rcx)
	jnc  .Lno_ssbd_\@
	orl  $FEATURE_ENABLE_SSBD, %eax
.Lno_ssbd_\@:
	movl $MSR_IA32_SPEC_CTRL, %ecx
	wrmsr
.endm

.macro IBRS_EXIT
	testl $SPEC_CTRL_PCP_IBRS_EXIT, IBRS_ENABLED_PCP
	jz .Lskip_\@

	pushq %rax
	pushq %rcx
	pushq %rdx
	__IBRS_EXIT
	popq %rdx
	popq %rcx
	popq %rax

.Lskip_\@:
.endm

.macro IBRS_EXIT_RESTORE_CLOBBER save_reg:req
	testl $SPEC_CTRL_PCP_IBRS, IBRS_ENABLED_PCP
	jz .Lskip_\@

	cmpl $NO_IBRS_RESTORE, \save_reg
	je  .Lskip_\@

	movl $MSR_IA32_SPEC_CTRL, %ecx
	movl IBRS_HI32_PCP, %edx
	movl \save_reg, %eax
	wrmsr
	jmp .Lend_\@

.Lskip_\@:
	lfence
.Lend_\@:
.endm

.macro IBRS_EXIT_CLOBBER
	testl $SPEC_CTRL_PCP_IBRS_EXIT, IBRS_ENABLED_PCP
	jz .Lskip_\@

	__IBRS_EXIT

.Lskip_\@:
.endm

.macro CLEAR_R8_TO_R15
	xorq %r15, %r15
	xorq %r14, %r14
	xorq %r13, %r13
	xorq %r12, %r12
	xorq %r11, %r11
	xorq %r10, %r10
	xorq %r9, %r9
	xorq %r8, %r8
.endm

.macro CLEAR_R10_TO_R15
	xorq %r15, %r15
	xorq %r14, %r14
	xorq %r13, %r13
	xorq %r12, %r12
	xorq %r11, %r11
	xorq %r10, %r10
.endm

#else /* __ASSEMBLY__ */

#ifdef CONFIG_SPEC_CTRL
/*
 * avoid changing the checksum of kabi-listed symbols
 * due to previous opaque elements now becoming
 * defined earlier than expected by __GENKSYMS__
 */
#ifndef __GENKSYMS__
#include <linux/ptrace.h>
#endif
#include <asm/processor.h>
#include <asm/bitops.h>
#include <asm/msr.h>
#include <asm/pda.h>

extern void spec_ctrl_init(void);
extern void spec_ctrl_rescan_cpuid(void);
extern void spec_ctrl_cpu_init(void);
extern void spec_ctrl_save_msr(void);
extern void ssb_select_mitigation(void);


extern bool spec_ctrl_force_enable_ibrs(void);
extern bool spec_ctrl_cond_enable_ibrs(bool full_retpoline);
extern bool spec_ctrl_enable_ibrs_always(void);
extern bool spec_ctrl_force_enable_ibp_disabled(void);
extern bool spec_ctrl_cond_enable_ibp_disabled(void);
extern void spec_ctrl_enable_retpoline_ibrs_user(void);
extern void x86_amd_ssbd_enable(void);

/* The Intel SPEC CTRL MSR base value cache */
extern u64 x86_spec_ctrl_base;
extern u64 x86_amd_ls_cfg_ssbd_mask;

static inline u64 ssbd_tif_to_spec_ctrl(u64 tifn)
{
	BUILD_BUG_ON(TIF_SSBD < FEATURE_ENABLE_SSBD_SHIFT);
	return (tifn & _TIF_SSBD) >> (TIF_SSBD - FEATURE_ENABLE_SSBD_SHIFT);
}

static inline u64 ssbd_tif_to_amd_ls_cfg(u64 tifn)
{
	return (tifn & _TIF_SSBD) ? x86_amd_ls_cfg_ssbd_mask : 0ULL;
}

extern void speculative_store_bypass_update(void);

enum {
	IBRS_DISABLED,

	/* in host kernel, disabled in guest and userland */
	IBRS_ENABLED,

	/* in host kernel and host userland, disabled in guest */
	IBRS_ENABLED_ALWAYS,

	/* in host userland, disabled in kernel and guest */
	IBRS_ENABLED_USER,

	IBRS_MAX = IBRS_ENABLED_USER,
};

extern u64 x86_spec_ctrl_base;

static inline int cpu_has_spec_ctrl(void)
{
	if (rh_cpu_test_cap(X86_FEATURE_IBRS))
		return 1;

	/* rmb to prevent wrong speculation for security */
	rmb();
	return 0;
}

static bool ibrs_enabled_kernel(void)
{
	extern unsigned int ibrs_mode;

	return ibrs_mode == IBRS_ENABLED || ibrs_mode == IBRS_ENABLED_ALWAYS;
}

static inline bool retp_enabled(void)
{
	return rh_cpu_test_cap(X86_FEATURE_RETPOLINE);
}

static inline bool ibpb_enabled(void)
{
	return (rh_cpu_test_cap(X86_FEATURE_IBPB) &&
		(ibrs_enabled_kernel() || retp_enabled()));
}

/*
 * On VMENTER we must preserve whatever view of the SPEC_CTRL MSR
 * the guest has, while on VMEXIT we restore the kernel view. This
 * would be easier if SPEC_CTRL were architecturally maskable or
 * shadowable for guests but this is not (currently) the case.
 * Takes the guest view of SPEC_CTRL MSR as a parameter.
 */

/*
 * RHEL note: Upstream implements two new functions to handle this:
 *
 *	- extern void x86_set_guest_spec_ctrl(u64);
 *	- extern void x86_restore_kernel_spec_ctrl(u64);
 *
 * We already have the following two functions in RHEL so the
 * above are not included in the RHEL version of the backport.
 */

static inline void spec_ctrl_ibrs_on(void)
{
	if (ibrs_enabled_kernel()) {
		u64 spec_ctrl = x86_spec_ctrl_base|FEATURE_ENABLE_IBRS;

		if (boot_cpu_data.x86_vendor == X86_VENDOR_INTEL)
			spec_ctrl |= ssbd_tif_to_spec_ctrl(
					current_thread_info()->flags);

		wrmsrl(MSR_IA32_SPEC_CTRL, spec_ctrl);
	} else {
		/* rmb to prevent wrong speculation for security */
		rmb();
 }
}

static inline void spec_ctrl_ibrs_off(void)
{
	if (ibrs_enabled_kernel()) {
		u64 spec_ctrl = x86_spec_ctrl_base;

		if (boot_cpu_data.x86_vendor == X86_VENDOR_INTEL)
			spec_ctrl |= ssbd_tif_to_spec_ctrl(
					current_thread_info()->flags);

		wrmsrl(MSR_IA32_SPEC_CTRL, spec_ctrl);
	}
	/* rmb not needed when disabling IBRS */
}

/*
 * These functions are called before calling into firmware.  Firmware might
 * have indirect branches, so if we're running with retpolines, we need to
 * enable IBRS to protect the kernel from spectre v2.
 *
 * The 'ibrs_on' variable is used to prevent race conditions.  Otherwise, if
 * the admin disabled IBRS while a CPU was running in firmware, IBRS could get
 * stuck on indefinitely.
 *
 * There are still other race conditions possible, but they're generally not a
 * problem because they'll get corrected on the next kernel exit.
 */
static inline bool spec_ctrl_ibrs_on_firmware(void)
{
	bool ibrs_on = false;

	if (cpu_has_spec_ctrl() && retp_enabled() && !ibrs_enabled_kernel()) {
		u64 spec_ctrl = x86_spec_ctrl_base|FEATURE_ENABLE_IBRS;

		if (boot_cpu_data.x86_vendor == X86_VENDOR_INTEL)
			spec_ctrl |= ssbd_tif_to_spec_ctrl(
					current_thread_info()->flags);

		wrmsrl(MSR_IA32_SPEC_CTRL, spec_ctrl);
		ibrs_on = true;
	} else {
		/* rmb to prevent wrong speculation for security */
		rmb();
	}

	return ibrs_on;
}

static inline void spec_ctrl_ibrs_off_firmware(bool ibrs_on)
{
	if (ibrs_on) {
		u64 spec_ctrl = x86_spec_ctrl_base;

		if (boot_cpu_data.x86_vendor == X86_VENDOR_INTEL)
			spec_ctrl |= ssbd_tif_to_spec_ctrl(
					current_thread_info()->flags);

		wrmsrl(MSR_IA32_SPEC_CTRL, spec_ctrl);
	} else {
		/* rmb to prevent wrong speculation for security */
		rmb();
	}
}

static inline void spec_ctrl_ibpb(void)
{
	if (ibpb_enabled())
		wrmsrl(MSR_IA32_PRED_CMD, FEATURE_SET_IBPB);
}

static inline void spec_ctrl_ibpb_if_different_creds(struct task_struct *next)
{
	if (ibpb_enabled() &&
	    (!next || __ptrace_may_access(next, PTRACE_MODE_IBPB)))
			wrmsrl(MSR_IA32_PRED_CMD, FEATURE_SET_IBPB);
}

static __always_inline void __spec_ctrl_vm_ibrs(u64 vcpu_ibrs, bool vmenter)
{
	u64 host_ibrs = 0, val;
	bool write_spec_ctrl;

	if (ibrs_enabled_kernel()) {
		/*
		 * If IBRS is enabled for host kernel mode or
		 * host always mode we must set
		 * FEATURE_ENABLE_IBRS at vmexit.
		 */
		host_ibrs = FEATURE_ENABLE_IBRS;
	}

	if (boot_cpu_data.x86_vendor == X86_VENDOR_INTEL)
		host_ibrs |= ssbd_tif_to_spec_ctrl(current_thread_info()->flags);

	val = vmenter ? vcpu_ibrs : host_ibrs;
	write_spec_ctrl = (!vmenter && host_ibrs) || (vcpu_ibrs != host_ibrs);

	/*
	 * IBRS may have barrier semantics so it must be set to
	 * satisfy those semantics during vmexit.
	 */
	if (write_spec_ctrl)
		wrmsrl(MSR_IA32_SPEC_CTRL, x86_spec_ctrl_base|val);
	else
		/* rmb to prevent wrong speculation for security */
		rmb();
}

static inline void spec_ctrl_vmenter_ibrs(u64 vcpu_ibrs)
{
       if (cpu_has_spec_ctrl())
               __spec_ctrl_vm_ibrs(vcpu_ibrs, true);
}

static inline void __spec_ctrl_vmexit_ibrs(u64 vcpu_ibrs)
{
	__spec_ctrl_vm_ibrs(vcpu_ibrs, false);
}

static __always_inline void stuff_RSB(void)
{
	__asm__ __volatile__("		call 1f; pause;"
			     "1:	call 2f; pause;"
			     "2:	call 3f; pause;"
			     "3:	call 4f; pause;"
			     "4:	call 5f; pause;"
			     "5:	call 6f; pause;"
			     "6:	call 7f; pause;"
			     "7:	call 8f; pause;"
			     "8:	call 9f; pause;"
			     "9:	call 10f; pause;"
			     "10:	call 11f; pause;"
			     "11:	call 12f; pause;"
			     "12:	call 13f; pause;"
			     "13:	call 14f; pause;"
			     "14:	call 15f; pause;"
			     "15:	call 16f; pause;"
			     "16:	call 17f; pause;"
			     "17:	call 18f; pause;"
			     "18:	call 19f; pause;"
			     "19:	call 20f; pause;"
			     "20:	call 21f; pause;"
			     "21:	call 22f; pause;"
			     "22:	call 23f; pause;"
			     "23:	call 24f; pause;"
			     "24:	call 25f; pause;"
			     "25:	call 26f; pause;"
			     "26:	call 27f; pause;"
			     "27:	call 28f; pause;"
			     "28:	call 29f; pause;"
			     "29:	call 30f; pause;"
			     "30:	call 31f; pause;"
			     "31:	call 32f; pause;"
			     "32:	add $(32*8), %%rsp": : :"memory");
}

#else /* CONFIG_SPEC_CTRL */

static inline void spec_ctrl_enable_ibrs (void)		{ }
static inline void spec_ctrl_disable_ibrs(void)		{ }
static inline void spec_ctrl_ibpb	 (void)		{ }
static inline void spec_ctrl_rescan_cpuid(void)		{ }
static inline void spec_ctrl_cpu_init	 (void)		{ }
static inline int  cpu_has_spec_ctrl	 (void)		{ return 0; }

static inline void spec_ctrl_vmenter_ibrs (u64 vcpu_ibrs) { }
static inline void __spec_ctrl_vmexit_ibrs(u64 vcpu_ibrs) { }
static inline void __spec_ctrl_vm_ibrs	  (u64 vcpu_ibrs, bool vmenter) { }

static inline bool spec_ctrl_force_enable_ibrs(void)		{ return false; }
static inline bool spec_ctrl_cond_enable_ibrs(bool full_retpoline) { return false; }
static inline bool spec_ctrl_enable_ibrs_always(void)		{ return false; }
static inline bool spec_ctrl_force_enable_ibp_disabled(void)	{ return false; }
static inline bool spec_ctrl_cond_enable_ibp_disabled(void)	{ return false; }
static inline void spec_ctrl_enable_retpoline_ibrs_user(void)	{ }

static inline bool unprotected_firmware_begin(void)		{ return false; }
static inline void unprotected_firmware_end(bool ibrs_on)	{ }

#endif /* CONFIG_SPEC_CTRL */

#endif /* __ASSEMBLY__ */
#endif /* _ASM_X86_SPEC_CTRL_H */
