/*
 *  include/asm-x86_64/bugs.h
 *
 *  Copyright (C) 1994  Linus Torvalds
 *  Copyright (C) 2000  SuSE
 *
 * This is included by init/main.c to check for architecture-dependent bugs.
 *
 * Needs:
 *	void check_bugs(void);
 */

#include <asm/processor.h>
#include <asm/i387.h>
#include <asm/msr.h>
#include <asm/pda.h>
#include <asm-generic/spec_ctrl.h>
#include <asm/msr.h>

extern void alternative_instructions(void);
extern void spectre_v2_select_mitigation(void);
extern void ssb_select_mitigation(void);
extern void spec_ctrl_save_msr(void);

#ifdef CONFIG_SPEC_CTRL
extern u64 x86_spec_ctrl_base;
#endif

static void __init check_bugs(void)
{
	identify_cpu(&boot_cpu_data);
#if !defined(CONFIG_SMP)
	printk("CPU: ");
	print_cpu_info(&boot_cpu_data);
#endif

	/*
	 * Select proper mitigation for any exposure to the Speculative Store
	 * Bypass vulnerability (exposed as a bug in "Memory Disambiguation")
	 * This has to be done before spec_ctrl_init() to make sure that its
	 * SPEC_CTRL MSR value is properly set up.
	 */
	ssb_select_mitigation();

	/* Select the proper spectre mitigation before patching alternatives */
#ifdef CONFIG_SPEC_CTRL
	/*
	 * If the x86_spec_ctrl_base is modified, propagate it to the
	 * percpu spec_ctrl structure.
	 */
	if (x86_spec_ctrl_base) {
		/*
		 * in RHEL6 wrmsrl and spec_ctrl_save_msr is done in
		 * spec_ctrl_init
		 */
		wrmsrl(MSR_IA32_SPEC_CTRL, x86_spec_ctrl_base);
		spec_ctrl_save_msr();
	}
#endif
	spectre_v2_select_mitigation();

	spec_ctrl_cpu_init();

	alternative_instructions(); 
}
