/*
 * cpufeature.h
 *
 * Defines x86 CPU feature bits
 */

#ifndef __ASM_X8664_CPUFEATURE_H
#define __ASM_X8664_CPUFEATURE_H

#define NCAPINTS	7	/* N 32-bit words worth of info */

/* Intel-defined CPU features, CPUID level 0x00000001, word 0 */
#define X86_FEATURE_FPU		(0*32+ 0) /* Onboard FPU */
#define X86_FEATURE_VME		(0*32+ 1) /* Virtual Mode Extensions */
#define X86_FEATURE_DE		(0*32+ 2) /* Debugging Extensions */
#define X86_FEATURE_PSE 	(0*32+ 3) /* Page Size Extensions */
#define X86_FEATURE_TSC		(0*32+ 4) /* Time Stamp Counter */
#define X86_FEATURE_MSR		(0*32+ 5) /* Model-Specific Registers, RDMSR, WRMSR */
#define X86_FEATURE_PAE		(0*32+ 6) /* Physical Address Extensions */
#define X86_FEATURE_MCE		(0*32+ 7) /* Machine Check Architecture */
#define X86_FEATURE_CX8		(0*32+ 8) /* CMPXCHG8 instruction */
#define X86_FEATURE_APIC	(0*32+ 9) /* Onboard APIC */
#define X86_FEATURE_SEP		(0*32+11) /* SYSENTER/SYSEXIT */
#define X86_FEATURE_MTRR	(0*32+12) /* Memory Type Range Registers */
#define X86_FEATURE_PGE		(0*32+13) /* Page Global Enable */
#define X86_FEATURE_MCA		(0*32+14) /* Machine Check Architecture */
#define X86_FEATURE_CMOV	(0*32+15) /* CMOV instruction (FCMOVCC and FCOMI too if FPU present) */
#define X86_FEATURE_PAT		(0*32+16) /* Page Attribute Table */
#define X86_FEATURE_PSE36	(0*32+17) /* 36-bit PSEs */
#define X86_FEATURE_PN		(0*32+18) /* Processor serial number */
#define X86_FEATURE_CLFLSH	(0*32+19) /* Supports the CLFLUSH instruction */
#define X86_FEATURE_DTES	(0*32+21) /* Debug Trace Store */
#define X86_FEATURE_ACPI	(0*32+22) /* ACPI via MSR */
#define X86_FEATURE_MMX		(0*32+23) /* Multimedia Extensions */
#define X86_FEATURE_FXSR	(0*32+24) /* FXSAVE and FXRSTOR instructions (fast save and restore */
				          /* of FPU context), and CR4.OSFXSR available */
#define X86_FEATURE_XMM		(0*32+25) /* Streaming SIMD Extensions */
#define X86_FEATURE_XMM2	(0*32+26) /* Streaming SIMD Extensions-2 */
#define X86_FEATURE_SELFSNOOP	(0*32+27) /* CPU self snoop */
#define X86_FEATURE_HT		(0*32+28) /* Hyper-Threading */
#define X86_FEATURE_ACC		(0*32+29) /* Automatic clock control */
#define X86_FEATURE_IA64	(0*32+30) /* IA-64 processor */

/* AMD-defined CPU features, CPUID level 0x80000001, word 1 */
/* Don't duplicate feature flags which are redundant with Intel! */
#define X86_FEATURE_SYSCALL	(1*32+11) /* SYSCALL/SYSRET */
#define X86_FEATURE_MMXEXT	(1*32+22) /* AMD MMX extensions */
#define X86_FEATURE_FXSR_OPT	(1*32+25) /* FXSR optimizations */
#define X86_FEATURE_RDTSCP	(1*32+27) /* RDTSCP */
#define X86_FEATURE_LM		(1*32+29) /* Long Mode (x86-64) */
#define X86_FEATURE_3DNOWEXT	(1*32+30) /* AMD 3DNow! extensions */
#define X86_FEATURE_3DNOW	(1*32+31) /* 3DNow! */

/* Transmeta-defined CPU features, CPUID level 0x80860001, word 2 */
#define X86_FEATURE_RECOVERY	(2*32+ 0) /* CPU in recovery mode */
#define X86_FEATURE_LONGRUN	(2*32+ 1) /* Longrun power control */
#define X86_FEATURE_LRTI	(2*32+ 3) /* LongRun table interface */

/* Other features, Linux-defined mapping, word 3 */
/* This range is used for feature bits which conflict or are synthesized */
#define X86_FEATURE_CXMMX	(3*32+ 0) /* Cyrix MMX extensions */
#define X86_FEATURE_K6_MTRR	(3*32+ 1) /* AMD K6 nonstandard MTRRs */
#define X86_FEATURE_CYRIX_ARR	(3*32+ 2) /* Cyrix ARRs (= MTRRs) */
#define X86_FEATURE_CENTAUR_MCR	(3*32+ 3) /* Centaur MCRs (= MTRRs) */
#define X86_FEATURE_REP_GOOD	(3*32+ 4) /* rep microcode works well on this CPU */
#define X86_FEATURE_CONSTANT_TSC (3*32+5) /* TSC runs at constant rate */
#define X86_FEATURE_SYNC_RDTSC  (3*32+6)  /* RDTSC syncs CPU core */
#define X86_FEATURE_FXSAVE_LEAK (3*32+7)  /* FIP/FOP/FDP leaks through FXSAVE */
#define X86_FEATURE_UP		(3*32+8) /* SMP kernel running on UP */
#define X86_FEATURE_ARCH_PERFMON (3*32+9) /* Intel Architectural PerfMon */
#define X86_FEATURE_IDA		(3*32+16) /* Intel Dynamic Acceleration */

#define X86_FEATURE_LFENCE_RDTSC (3*32+18) /* Lfence synchronizes RDTSC */
#define X86_FEATURE_PCI_EXT_CFG	(3*32+19) /* PCI extended cfg access */
#define X86_FEATURE_TSC_RELIABLE (3*32+23) /* TSC is known to be reliable */
#define X86_FEATURE_NONSTOP_TSC (3*32+24) /* TSC does not stop in C states */
#define X86_FEATURE_ARAT	(3*32+25) /* Always Running APIC Timer */
#define X86_FEATURE_AMD_DCM	(3*32+27) /* multi-node processor */
#define X86_FEATURE_CLFLUSH_MONITOR (3*32+28) /* clflush reqd with monitor */
#define X86_FEATURE_CPB		(3*32+29) /* AMD Core Performance Boost */

/* Intel-defined CPU features, CPUID level 0x00000001 (ecx), word 4 */
#define X86_FEATURE_XMM3	(4*32+ 0) /* Streaming SIMD Extensions-3 */
#define X86_FEATURE_MWAIT	(4*32+ 3) /* Monitor/Mwait support */
#define X86_FEATURE_DSCPL	(4*32+ 4) /* CPL Qualified Debug Store */
#define X86_FEATURE_EST		(4*32+ 7) /* Enhanced SpeedStep */
#define X86_FEATURE_TM2		(4*32+ 8) /* Thermal Monitor 2 */
#define X86_FEATURE_SSSE3	(4*32+ 9) /* Supplemental SSE-3 */
#define X86_FEATURE_CID		(4*32+10) /* Context ID */
#define X86_FEATURE_FMA		(4*32+12) /* Fused multiply-add */
#define X86_FEATURE_CX16	(4*32+13) /* CMPXCHG16B */
#define X86_FEATURE_XTPR	(4*32+14) /* Send Task Priority Messages */
#define X86_FEATURE_DCA		(4*32+18) /* Direct Cache Access */
#define X86_FEATURE_XMM4_1	(4*32+19) /* "sse4_1" SSE-4.1 */
#define X86_FEATURE_XMM4_2	(4*32+20) /* "sse4_2" SSE-4.2 */
#define X86_FEATURE_X2APIC	(4*32+21) /* x2APIC */
#define X86_FEATURE_MOVBE	(4*32+22) /* MOVBE instruction */
#define X86_FEATURE_POPCNT      (4*32+23) /* POPCNT instruction */
#define X86_FEATURE_XSAVE	(4*32+26) /* XSAVE/XRSTOR/XSETBV/XGETBV */
#define X86_FEATURE_OSXSAVE	(4*32+27) /* "" XSAVE enabled in the OS */
#define X86_FEATURE_AVX		(4*32+28) /* Advanced Vector Extensions */
#define X86_FEATURE_RDRAND	(4*32+30) /* The RDRAND instruction */
#define X86_FEATURE_HYPERVISOR	(4*32+31) /* Running on a hypervisor */

/* VIA/Cyrix/Centaur-defined CPU features, CPUID level 0xC0000001, word 5 */
#define X86_FEATURE_XSTORE	(5*32+ 2) /* on-CPU RNG present (xstore insn) */
#define X86_FEATURE_XSTORE_EN	(5*32+ 3) /* on-CPU RNG enabled */
#define X86_FEATURE_XCRYPT	(5*32+ 6) /* on-CPU crypto (xcrypt insn) */
#define X86_FEATURE_XCRYPT_EN	(5*32+ 7) /* on-CPU crypto enabled */

/* More extended AMD flags: CPUID level 0x80000001, ecx, word 6 */
#define X86_FEATURE_LAHF_LM	(6*32+ 0) /* LAHF/SAHF in long mode */
#define X86_FEATURE_CMP_LEGACY	(6*32+ 1) /* If yes HyperThreading not valid */
#define X86_FEATURE_SVM		(6*32+ 2) /* Secure virtual machine */
#define X86_FEATURE_EXTAPIC	(6*32+ 3) /* Extended APIC space */
#define X86_FEATURE_CR8_LEGACY	(6*32+ 4) /* CR8 in 32-bit mode */
#define X86_FEATURE_ABM		(6*32+ 5) /* Advanced bit manipulation */
#define X86_FEATURE_SSE4A	(6*32+ 6) /* SSE-4A */
#define X86_FEATURE_MISALIGNSSE (6*32+ 7) /* Misaligned SSE mode */
#define X86_FEATURE_3DNOWPREFETCH (6*32+ 8) /* 3DNow prefetch instructions */
#define X86_FEATURE_OSVW	(6*32+ 9) /* OS Visible Workaround */
#define X86_FEATURE_IBS		(6*32+10) /* Instruction Based Sampling */
#define X86_FEATURE_XOP		(6*32+11) /* extended AVX instructions */
#define X86_FEATURE_SKINIT	(6*32+12) /* SKINIT/STGI instructions */
#define X86_FEATURE_WDT		(6*32+13) /* Watchdog timer */
#define X86_FEATURE_LWP		(6*32+15) /* Light Weight Profiling */
#define X86_FEATURE_FMA4	(6*32+16) /* 4 operands MAC instructions */
#define X86_FEATURE_NODEID_MSR	(6*32+19) /* NodeId MSR */
#define X86_FEATURE_TBM		(6*32+21) /* trailing bit manipulations */
#define X86_FEATURE_TOPOEXT	(6*32+22) /* topology extensions CPUID leafs */
#define X86_FEATURE_PERFCTR_CORE (6*32+23) /* core perf counter extensions */
#define X86_FEATURE_PERFCTR_NB	(6*32+24) /* NB perf counter extensions */

/*
 * Auxiliary flags: Linux defined - For features scattered in various
 * CPUID levels like 0x6, 0xA etc
 *
 * Extended features CPUID (EAX=7, ECX=0) have to be kept on a
 * separated capability bitmap word on rhel5, in order to keep
 * kABI with minimal extentions
 *
 * Reuse free bits when adding new feature flags -- check CPUID query
 * table at arch/i386/kernel/cpu/addon_cpuid_features.c
 */

/* Intel-defined CPU features, CPUID level 0x00000007:0 (EDX), word 18 */

/* CPU features, CPUID level 0x80000008:0 (EDX, EBX), word 13 */
#define X86_FEATURE_SMEP		(7*32+3) /* Supervisor Mode Execution Protection */
#define X86_FEATURE_SPEC_CTRL		(7*32+4) /* "" Speculation Control (IBRS + IBPB) bit 26 */
#define X86_FEATURE_INTEL_STIBP         (7*32+5) /* Single Thread Indirect Branch Predictors bit 27 */
#define X86_FEATURE_ARCH_CAPABILITIES	(7*32+6) /* IA32_ARCH_CAPABILITIES MSR (Intel) bit 29 */
#define X86_FEATURE_IBPB		(7*32+7) /* Prediction Command MSR (AMD) bit 12 */
#define X86_FEATURE_IBRS		(7*32+8) /* Speculation Control MSR only bit 14 */
#define X86_FEATURE_STIBP		(7*32+10) /* Single Thread Indirect Branch Predictors (AMD) bit 15 */

/* virtual feature bits (e.g. not from cpuid instruction) */
#define X86_FEATURE_AMD_SSBD		(7*32+24) /* "" AMD RDS implementation */
#define X86_FEATURE_SSBD		(7*32+27) /* Reduced Data Speculation (formerly "MDD") */
#define X86_FEATURE_SPEC_STORE_BYPASS_DISABLE	(7*32+28) /* "" Disable Speculative Store Bypass. */
#define X86_FEATURE_PTI			(7*32+29) /* Kernel Page Table Isolation enabled */
#define X86_FEATURE_RETPOLINE		(7*32+30) /* Generic Retpoline mitigation for Spectre variant 2 */
#define X86_FEATURE_IBP_DISABLE		(7*32+31) /* Old AMD Indirect Branch Predictor Disable */

/* start of RH extended capabilities word 7, bit 0 */
#define RH_EXT_CAP_BIT0			(7*32+0)
#define RH_NCAPINTS			1

#define RHNCAPINTS	(NCAPINTS+RH_NCAPINTS)

#ifndef __ASSEMBLY__

/*
 * Set of macros to read/write the RH extended CPU capability bitmap
 *
 * Although we must keep the original CPUID leaf index to avoid
 * bit IDs overlapping with IDs from other leaves, we do not need
 * to carry the flags across all CPUs, as, basically, all the sets
 * and checks can be done on the consolidated bitmap boot_cpu_data.
 *
 * We're extending the CPU capability bitmap mostly to be able to cope
 * with speculative execution control flags (Spectre/Meltdown) capabilities,
 * so in order to avoid convoluted code complexity and further kABI-hiding
 * __GENKSYMS__ dances, we're going to emulate boot_cpu_data.x86_capability
 * bitmap via the external variable __cpu_smelt_flags, and providing
 * separated accessors to it, that will be used in code sites accordingly.
 *
 * As we're maintaining the original CPUID leaf index for the extended bits
 * but only have one word worthy of bitmap storage, we need to offset
 * the bit by -RH_EXT_CAP_BIT0 when operating against the bitmap
 */
extern unsigned long __cpu_smelt_flags;
#define __rh_cpu_cap_fn(func, bit)					\
	func##_bit((bit-RH_EXT_CAP_BIT0), (unsigned long *)(&__cpu_smelt_flags))

#define rh_cpu_set_cap(bit)		__rh_cpu_cap_fn(set, bit)
#define rh_cpu_test_cap(bit)		__rh_cpu_cap_fn(test, bit)
#define rh_cpu_clear_cap(bit)		__rh_cpu_cap_fn(clear, bit)

/*
 * CPU bug word emulation
 */
extern unsigned long __cpu_bugs;
#define boot_cpu_has_bug(bit)		test_bit(bit, &__cpu_bugs)
#define boot_cpu_set_bug(bit)		set_bit(bit, &__cpu_bugs)
#define boot_cpu_clear_bug(bit)		clear_bit(bit, &__cpu_bugs)
#define setup_force_cpu_bug(bit)	boot_cpu_set_bug(bit)

extern __u32 cpu_caps_set[RHNCAPINTS];

/*
 * RHEL: If the bit is in the cpuinfo_x86 struct (ie, is word 8 or less),
 * then just use the bit like it always has been.  If it is in the new
 * extended struct, cpuinfo_x86_rh, then check to see if the cpuinfo_x86
 * struct was the boot_cpu_data struct.  If it was the boot_cpu_data
 * struct then use boot_cpu_data_rh, o/w use the cpu_data_rh per_cpu data
 */
#define _cpu_cap_fn(func, c, bit)					\
	((bit < RH_EXT_CAP_BIT0) ?					\
	 func##_bit(bit, (unsigned long *)((c)->x86_capability)) :	\
	 ((c == &boot_cpu_data) ?					\
	  func##_bit(bit - RH_EXT_CAP_BIT0,				\
		     (unsigned long *)boot_cpu_data_rh.x86_capability):	\
	  func##_bit(bit - RH_EXT_CAP_BIT0,				\
		 (unsigned long *)cpu_data_rh((c)->cpu_index).x86_capability)\
	 )								\
	)

#define test_cpu_cap(c, bit) _cpu_cap_fn(test, c, bit)
#define set_cpu_cap(c, bit) _cpu_cap_fn(set, c, bit)

#define setup_force_cpu_cap(bit) do { \
	rh_cpu_set_cap(&boot_cpu_data, bit);	\
	set_bit(bit, (unsigned long *)cpu_caps_set);	\
} while (0)

#define X86_BUG(x)			(x)
#define X86_BUG_CPU_MELTDOWN		X86_BUG(0) /* CPU is affected by meltdown attack and needs kernel page table isolation */
#define X86_BUG_SPECTRE_V1		X86_BUG(1) /* CPU is affected by Spectre variant 1 attack with conditional branches */
#define X86_BUG_SPECTRE_V2		X86_BUG(2) /* CPU is affected by Spectre variant 2 attack with indirect branches */
#define X86_BUG_SPEC_STORE_BYPASS	X86_BUG(3) /* CPU is affected by speculative store bypass attack */

#define cpu_has(c, bit)                test_bit(bit, (c)->x86_capability)
#define boot_cpu_has(bit)      test_bit(bit, boot_cpu_data.x86_capability)
#define clear_cpu_cap(c, bit)  clear_bit(bit, (unsigned long *)((c)->x86_capability))

extern __u32 cpu_caps_set[RHNCAPINTS];

/*
 * RHEL: If the bit is in the cpuinfo_x86 struct (ie, is word 8 or less),
 * then just use the bit like it always has been.  If it is in the new
 * extended struct, cpuinfo_x86_rh, then check to see if the cpuinfo_x86
 * struct was the boot_cpu_data struct.  If it was the boot_cpu_data
 * struct then use boot_cpu_data_rh, o/w use the cpu_data_rh per_cpu data
 */
#define _cpu_cap_fn(func, c, bit)					\
	((bit < RH_EXT_CAP_BIT0) ?					\
	 func##_bit(bit, (unsigned long *)((c)->x86_capability)) :	\
	 ((c == &boot_cpu_data) ?					\
	  func##_bit(bit - RH_EXT_CAP_BIT0,				\
		     (unsigned long *)boot_cpu_data_rh.x86_capability):	\
	  func##_bit(bit - RH_EXT_CAP_BIT0,				\
		 (unsigned long *)cpu_data_rh((c)->cpu_index).x86_capability)\
	 )								\
	)

#define test_cpu_cap(c, bit) _cpu_cap_fn(test, c, bit)
#define set_cpu_cap(c, bit) _cpu_cap_fn(set, c, bit)

#define setup_force_cpu_cap(bit) do { \
	rh_cpu_set_cap(&boot_cpu_data, bit);	\
	set_bit(bit, (unsigned long *)cpu_caps_set);	\
} while (0)

#define cpu_has_fpu            1
#define cpu_has_vme            0
#define cpu_has_de             1
#define cpu_has_pse            1
#define cpu_has_tsc            1
#define cpu_has_pae            ___BUG___
#define cpu_has_pge            1
#define cpu_has_apic           boot_cpu_has(X86_FEATURE_APIC)
#define cpu_has_mtrr           1
#define cpu_has_mmx            1
#define cpu_has_fxsr           1
#define cpu_has_xmm            1
#define cpu_has_xmm2           1
#define cpu_has_xmm3           boot_cpu_has(X86_FEATURE_XMM3)
#define cpu_has_ht             boot_cpu_has(X86_FEATURE_HT)
#define cpu_has_mp             1 /* XXX */
#define cpu_has_k6_mtrr        0
#define cpu_has_cyrix_arr      0
#define cpu_has_centaur_mcr    0
#define cpu_has_clflush	       boot_cpu_has(X86_FEATURE_CLFLSH)
#define cpu_has_arch_perfmon	boot_cpu_has(X86_FEATURE_ARCH_PERFMON)
#define cpu_has_hypervisor	boot_cpu_has(X86_FEATURE_HYPERVISOR)
#define cpu_has_xmm4_1		boot_cpu_has(X86_FEATURE_XMM4_1)
#define cpu_has_xmm4_2		boot_cpu_has(X86_FEATURE_XMM4_2)
#define cpu_has_pci_ext_cfg	boot_cpu_has(X86_FEATURE_PCI_EXT_CFG)

#endif /* !__ASSEMBLY__ */

#endif /* __ASM_X8664_CPUFEATURE_H */
